package Conf;

#
# Configuration management :
# read the options on the command-line and in config files
#
# Export the hash %Conf for accessing to the configuration
#

require Exporter;
use Getopt::Long;
use Carp qw(carp croak);
use strict;
use vars qw(@ISA @EXPORT %Conf %Conf_default);

@ISA = qw(Exporter);
@EXPORT = qw(%Conf);

#
# All the options available to the command-line
# 
my @first_opt = qw(config_file=s home_config_file=s);
my @options = qw(data_dir=s nls_dir=s db_access=s db_access_options=s
		 market_dir=s logo=s height=i width=i 
                 candle_width=i zone3fac=f zone2fac=f indicators=s);

#
# Built-in default values
#
%Conf =
( 'data_dir'     => '/var/spool/geniustrader',
  'market_dir'   => '/var/spool/geniustrader/markets',
  'config_file'  => '/etc/geniustrader.conf',
  'home_config_file' => '.geniustrader/config',
  'nls_dir'      => '/usr/share/geniustrader/nls',
  'db_access'    => 'DB::Local',
  'db_access_options' => '/var/spool/geniustrader',
  'logo'	 => '/usr/share/geniustrader/logo.xpm',
  'height'	 => 480,
  'width'	 => 640,
  'candle_width' => 4,
  'zone3fac'     => 0.15,
  'zone2fac'     => 0.20,
  'indicators'   => 'COURS,VOLUME,AMA-100'
);

%Conf_default = %Conf;

#
# Load data from command-line options and/or from config file
#
sub load {
 # Reinitialize default values
 %Conf = %Conf_default;
 # Read important command-line options
 Getopt::Long::config('pass_through');
 GetOptions(\%Conf, @first_opt);
 Getopt::Long::config('no_pass_through');

 # Parse the config file !
 if (-f $Conf{'config_file'}) {
   parse_file($Conf{'config_file'});
 }
 if (-f "$ENV{'HOME'}/$Conf{'home_config_file'}") {
   parse_file("$ENV{'HOME'}/$Conf{'home_config_file'}");
 }
 
 # Read other command-line options
 GetOptions(\%Conf, @options);
}

sub parse_file {
  my $file = shift;
  my ($key, $value);
  open(CFG, '<' . $file) ||
    die "Can't open config file $file : $!\n";
  while (defined($_=<CFG>)) {
    chomp;
    next if /^\s*(?:#|$)/;
    if (m/^\s*(\S+)\s+(\`[^\`]+\`|\S+)\s*$/) {
       $key = lc($1); $value= $2;
       $key =~ tr/-/_/; # Special feature
       $value = qx/$1/ if ($value =~ /\`(.*)\`/);
       chomp($value);
       $Conf{$key} = $value;
    } else {
      carp "Line $. with bad syntax in $file : '$_'\n";
    }
  }
  close CFG;
}

1;

__END__

=head1 Conf.pm file

This module export a %Conf hash in order to allow you to access to
the Geniustrader options. The %Conf hash can be updated by running
Conf::load() if necessary. The %Conf hash takes his values from
variours sources : /etc/geniustrader.conf, ~/.geniustrader/config or
the command line options.

=cut
