#!/usr/bin/perl -w

package LocalDB;

use MLDBM qw(DB_File Storable);
use DB_File;
use Carp;
use Fcntl;
use Exporter;
use strict;
use vars qw(@ISA @EXPORT @EXPORT_OK);

@ISA = qw();
@EXPORT = qw();
@EXPORT_OK = qw(key_exists new open set get delete set_keyname);

## General comments ##
#
# All files are searched in the current directory, so you
# have to chdir before calling new/open/exists ...
# Or you can give an $id that is a complete filename.
#
## General comments ##

# For a new key => create the database
sub new {
  my $this = shift;
  my $class = ref($this) || $this;
  my $id = lc(shift) || return undef;
  my $self = { 'id' => $id, 'db' => {} };
  bless $self, $class;
  ## Creating the database
  $self->{'dbref'} = tie %{$self->{'db'}}, 'MLDBM', $id,  O_CREAT|O_RDWR, 
   0664, $DB_BTREE or croak "Cannot create/open $id database: $!\n";
  $self->{'dbref'}->DumpMeth('portable');
  return $self;
}

# For an already existing key => open the database
sub open {
  my $this = shift;
  my $class = ref($this) || $this;
  my $id = lc(shift) || return undef; ## Key id
  my $write = shift;              ## True if write mode is needed
  my $flags = $write ? O_RDWR : O_RDONLY;
  my $self = { 'id' => $id, 'db' => {} };
  bless $self, $class;
  ## Opening the database
  $self->{'dbref'} = tie %{$self->{'db'}}, 'MLDBM', $id, $flags, 0664,
   $DB_BTREE or croak "Cannot open $id database: $!\n";
  $self->{'dbref'}->DumpMeth('portable');
  return $self;
}

# Check if a key already exist
sub key_exists {
  my $id = shift;
  chomp($id);
  return ( -f $id || -f "$id.db" ? 1 : 0);
}

# Some cleanups when destroying
sub DESTROY {
  my $self = shift;
  $self->{'dbref'} = undef;
  untie %{$self->{'db'}}; # if (tied %{$self->{'db'}});
}

# Adding a value
sub set {
  my $self = shift;
  my $tab = shift; # $tab = [ date, open, high, low, close, volume ]

  prepare_data($tab);

  $self->{'db'}->{$tab->[0]} = [ @{$tab}[1..5] ];
}

# Get a value
sub get {
  my $self = shift;
  my $date = shift;
  
  if (exists $self->{'db'}{$date}) {
    return $self->{'db'}{$date};
  } else { return undef };
}

# Delete a value
sub delete {
  my $self = shift;
  my $date = shift;

  delete $self->{'db'}{$date};
}

# Formats data correctly before writing them to the database
sub prepare_data {
  my $tab = shift;
  foreach (1..5) { $tab->[$_] = $tab->[$_] + 0; }
}

# Setup a correspondance key <-> value name
sub set_keyname {
  my $id = lc(shift);
  my $name = lc(shift);
  my %index;
  
  $name =~ s/\b(.)/\U$1/g;
  
  tie %index, 'DB_File', 'index', O_CREAT|O_RDWR, 0664, $DB_BTREE
   or croak "Cannot access index database in write mode: $!\n";
  $index{$id} = $name;
  untie %index;
}

1;

__END__

=head1 LocalDB.pm

This module provides an object for creating/modifying a quotes database for
a precise value (aka id). 

=cut
