package Market;

#
# Manage markets
#

use Conf;
use Carp qw(croak carp);
use strict;

#
# Create a new market
#
sub new {
  my $self = shift;
  my $class = ref($self) || $self;
  my $code = lc(shift);
  my $name = shift;
  my $node = { 'code' => $code, 
               'name' => $name, 
	       'filename' => "$Conf::Conf{'market_dir'}/$code.mar",
	       'list' => [ ]
	     };
  open(MARKET, ">$node->{'filename'}") || 
    croak "Can't open $node->{'filename'} in write mode : $!\n";
  print MARKET "$name\n";
  close MARKET;
  return bless $node, $class;
}

#
# Open a market
#
sub open_market {
  my $self = shift;
  my $class = ref($self) || $self;
  my $code = lc(shift);
  my $node = { 'code' => $code,
               'filename' => "$Conf{'market_dir'}/$code.mar",
	       'name' => 'no_value',
	       'list' => [ ]
             };
  open(MARKET, "<$node->{'filename'}") ||
    croak "Can't open $node->{'filename'} for reading : $!\n";
  chomp($node->{'name'} = <MARKET>);
  while(defined($_=<MARKET>)) {
    chomp;
    push(@{$node->{'list'}}, $_) if ($_ ne '');
  }
  close MARKET;
  return bless $node, $class;
}

#
# Write down any modifications
#
sub write {
  my $self = shift;
  
  open (MARKET, ">$self->{'filename'}") ||
    croak "Can't open $self->{'filename'} in write mode : $!\n";
  print MARKET "$self->{'name'}\n";
  print MARKET join("\n", grep(!/^$/, @{$self->{'list'}})) . "\n";
  close MARKET;
}

#
# Get the list of key ids
#
sub get_list {
  my $self = shift;

  grep (!/^$/, @{$self->{'list'}});
}

#
# Add a quote to a market
#
sub add {
  my $self = shift;
  my $id = lc(shift);

  push(@{$self->{'list'}}, $id) if (! grep(/^$id$/,@{$self->{'list'}}));
}

#
# Delete an element of the list
#
sub delete {
  my $self = shift;
  my $id = lc(shift);

  foreach (grep(/^$id$/, @{$self->{'list'}})) { $_ = '' };
}

sub name {
  my $self= shift;

  return $self->{'name'};
}

sub id {
  my $self = shift;

  return $self->{'code'};
}

#
# Get the complete list of markets
# Not part of the object
#
sub market_list {
  my @list = ();
  opendir(MARKET_LIST, $Conf{'market_dir'}) || 
    die "Can't open dir $Conf{'market_dir'} : $!\n";
  foreach (readdir MARKET_LIST) {
    if (/\.mar$/) { s/\.mar$//; push(@list, $_); }
  }
  closedir MARKET_LIST;
  return @list;
}

1;
